/*Alain COUPEL

Firmware for Master clock module

V 1.1 for PIC 16F727

RA0 	ANALOG	IN	ANALOG
RA1 	FIVEV	IN	D
RA2		NC		OUT	D
RA3		NC		OUT	D	
RA4		NC		OUT	D	
RA5		NC		OUT	D	
RA6		CLK	
RA7		CLK

TRISA = 0b00000011;

RB0		ENC_A		IN
RB1		ENC_LED		OUT
RB2		ENC_B		IN
RB3		ENC_S 		IN
RB4		MID_SW		IN
RB5		BPM_SW		IN
RB6		PGC			OUT
RB7		PGD			OUT

TRISB = 0b00111101;

RC0		DISP_3		OUT
RC1		DISP_1		OUT
RC2		DISP_2		OUT
RC3		NC			OUT
RC4		BPM_LED		OUT
RC5		NC			OUT
RC6		NC			OUT
RC7		RX			IN

TRISC = 0b10000000;

RD0		DISP_g		OUT
RD1		DISP_a		OUT
RD2		DISP_b		OUT
RD3		DISP_f		OUT
RD4		DISP_c		OUT
RD5		DISP_d		OUT
RD6		DISP_e		OUT
RD7		Dot			OUT

TRISD = 0b00000000;

RE0		NC		OUT	D
RE1		NC		OUT	D
RE2		NC		OUT	D
RE3		MCLR	IN

TRISE = 0b00001000;

*/

 #include <htc.h> // Hi-TECH Compiler v 9.81
 
__CONFIG(FOSC_HS & WDTE_OFF & nPWRTE_OFF & MCLRE_ON & CP_OFF & BOREN_OFF & DEBUG_ON);
__CONFIG(VCAPEN_DIS); //__CONFIG(VCAPEN_RA5);

#define FIVEV RA1
 
#define ENCODER_A RB0
#define	ENCODER_LED RB1
#define	ENCODER_B RB2
#define ENCODER_SW RB3
#define SWING RB4
#define MIDI RB5

#define DISP1 RC1
#define DISP2 RC2
#define DISP3 RC0 
#define BPM_LED RC4

#define Disp_a RD1
#define	Disp_b RD2
#define	Disp_c RD4
#define	Disp_d RD5
#define	Disp_e RD6
#define	Disp_f RD3
#define	Disp_g RD0
#define Disp_Dot RD7

#define OUT_1 RE2	//X1 Output

volatile bit
Flag_INT;
	 
volatile unsigned char
Interrupt_Time,
Dot,
Beat,
ENCODER_F,
ENCSW_F,
Flag_INC,
INC_BPM,
PORT_OUT,
DISP_N,
Digit1,
Digit2,
Digit3,
MIDI_SYNC_VAL,
MIDI_SYNC_VAL_Old,
Rx_DATA,
nb_MIDI_Clocks;

volatile signed int
Loop_LED,			//time for the led is ON
Loop_BPM,			//Loop for BPM and Analog values
BPM_VAL,			// BPM_VAL = 2*BPM
BPM_VAL_ENC,		//BPM value given by the encoder
BPM_VAL_Old;		//Old value to see if BPM has changed

volatile signed int
ANALOG_VAL;

volatile unsigned long
Swing_VAL,
INC_Cycles,				//nb de cycle  dcrmenter  = 
Instructions_VAL,
nb_Instructions;

const unsigned char DisplayConvert[14] ={0b01111110, 0b00010100, 0b01100111, 0b00110111, 0b00011101, 0b00111011, 
	0b01111011, 0b00010110,0b01111111, 0b00111111,0b01011111, 0b01101010, 0b01101011, 0b00000000};
	

const unsigned long
BPM = 150000000; 
	 
void InitPIC(void){
	
	TRISA = 0b00000011;
	TRISB = 0b00111101;
	TRISC = 0b10000000;
	TRISD = 0b00000000;
	TRISE = 0b00001000;
	
	ANSELA = 0b00000001;	//activation des convertisseurs A/N	sur RA0	
	ANSELB = 0; 
	ANSELE = 0;
	
	ADCON0 = 1;				//ADC enabled				
	ADCON1 = 0b00100000;		// Fosc/32		
	PEIE = 1;		//Enables all unmasked peripheral interrupts
	RCIE = 0;		//disables Rx interrupt
	OPTION_REG = 0b10000000;
}
			
void InitSerial(void){		
	SPBRG=39;			//Value for 31250 bauds:((CLK/(16 * BAUD) -1))
	BRGH=1;				//High Speed
	SYNC=0;				//asynchronous
	SPEN=1;				//enable serial port pin
	CREN=1;				//enable reception
	TXEN = 0;			//Transmit disabled
	SREN=0;				//no effect
	RX9=0;				//8-bit reception
}

void InitTimer(void){
	T1CON = 0b01100101;	
	TMR1ON = 1;			//Restart Timer1	
}

void InitValues(void){	
	BPM_VAL_ENC = 240;
	BPM_VAL = 240;
	BPM_VAL_Old = 0;
	nb_Instructions = 156250; //BPM = 120
	INC_Cycles = nb_Instructions;
	Loop_LED = 400;
	Loop_BPM = 500;
	nb_MIDI_Clocks = 0;
	PORTE &= 0b11111000;
	INC_BPM = 1;
	ENCODER_LED = 0;
	BPM_LED = 0;
	MIDI_SYNC_VAL = 48;
	MIDI_SYNC_VAL_Old = 0;
		
}

void ShortDelay(void){ //705 instruction Cycles  (352 S at 8 MHZ)
	char i, j;
	for(i=0;i<40;i++)
	for(j=0;j<6;j++);
}	

void clear_usart_errors(void){
	if (OERR){																											
	CREN=0;													
	CREN=1;																											
	}															
	if (FERR){															
	Rx_DATA=RCREG;											
	TXEN=0;													
	TXEN=1;												
	}
}
char bitRead(int entier, char pos){
	if(entier & (1<< pos)){
		return 1;
	}
	else
	{
		return 0;
	}
}

int ReadCV(void){
	GO = 1;
	while(GO) continue;
	return (ADRES<<2);
}			

void DigitConvert(unsigned short LED_Val){
	LED_Val = LED_Val >> 1;	
	Digit1 = LED_Val/100;
	Digit2 =(LED_Val - Digit1*100)/10;
	Digit3 = LED_Val -Digit1*100 - Digit2 * 10;
	
	if(bitRead(BPM_VAL,0))Dot = 0x80;
	else Dot = 0;
	
	if(Digit1 == 0){
		Digit1 = 13;
		if(Digit2 == 0) Digit2 = 13;
	}		
}
//**************************ISR********************************	
void interrupt Timer(void){	
	
	TMR1ON = 0;			//Stop Timer1	
	TMR1IF = 0;			//clear interrupt flag bit 	
	TMR1H = 255;		// TMR1L must not be initialized
	TMR1L += 20;
	Interrupt_Time = 209 + TMR1L; //243				
	TMR1ON = 1;			//Restart Timer1
	Flag_INT = 1;		
}
//**********************END of ISR**************************
void main(void){
	int n;
	InitPIC();
	InitSerial();
	InitTimer();
	InitValues();
	
//********Display "ACS" ***********************

	for(n=0;n<8000;n++){	
		 switch(DISP_N){
			case 0:
				DISP3 = 0;
		 		PORTD = DisplayConvert[10]; // Displays "A"
				DISP1 =1;
			break;
			case 1:
				DISP1 =0;
		 		PORTD = DisplayConvert[11]; //displays "C"
				DISP2 =1;
			break;
			case 2:
				DISP2 =0;
		 		PORTD = DisplayConvert[5]; //Displays "S"
				DISP3 =1;
			break;
				
			}	
	ShortDelay();
		DISP_N++;
		if(DISP_N > 2) DISP_N = 0;
	}
	
	DigitConvert(BPM_VAL);  //Display BPM_VALUE (120)			

	TMR1H = 255;				
	TMR1L = 0; 			
	TMR1ON = 1;			//Restart Timer1			
	GIE = 1;			//enables interrupts


	while(1){
		
		if(MIDI){
			TMR1IE = 0;		//disables TMR1 Interrupts
			BPM_VAL_Old = 0;
			
			if(!ENCODER_A){
				if(ENCODER_F){
					if(ENCODER_B)MIDI_SYNC_VAL+=2;
					else MIDI_SYNC_VAL-=2;
					
					if(MIDI_SYNC_VAL > 96 ) MIDI_SYNC_VAL = 96;
					if(MIDI_SYNC_VAL < 4) MIDI_SYNC_VAL = 4;	
					}		
				ENCODER_F = 0;
				}
			else ENCODER_F = 1;
			
			if(MIDI_SYNC_VAL != MIDI_SYNC_VAL_Old){
				MIDI_SYNC_VAL_Old = MIDI_SYNC_VAL;			
				DigitConvert(MIDI_SYNC_VAL);
			}	
			
			if(RCIF){
				clear_usart_errors();
  		   		Rx_DATA=RCREG;
				if(Rx_DATA == 0xF8) nb_MIDI_Clocks--;
					 			  
				if(nb_MIDI_Clocks == 0){
					PORT_OUT++;
					if(PORT_OUT == 5) PORT_OUT = 1;	
					PORTE = PORT_OUT;
					BPM_LED = OUT_1;				
					nb_MIDI_Clocks = MIDI_SYNC_VAL >> 1;
				}                   
			}//if(RCIF			
									
		}
		else{
			TMR1IE = 1;		//enables TMR1 Interrupts
			MIDI_SYNC_VAL_Old = 0;
			
			if(!ENCODER_A){
					if(ENCODER_F){
						if(ENCODER_B)BPM_VAL_ENC +=INC_BPM;
						else BPM_VAL_ENC -=INC_BPM;
						
						if(BPM_VAL_ENC < 2) BPM_VAL_ENC = 2;
						if(BPM_VAL_ENC > 1999) BPM_VAL_ENC = 1999;	
						}		
					ENCODER_F = 0;
			}
			else ENCODER_F = 1;	
						
			if(!ENCODER_SW){
					if(ENCSW_F){
					Flag_INC = !Flag_INC;
					ENCODER_LED = 0;
					INC_BPM = 1;
					if(Flag_INC){
						ENCODER_LED = 1;
						INC_BPM = 20;
					}	
					ENCSW_F = 0;
					}
					}
				else ENCSW_F = 1;
					
				
					if(BPM_VAL != BPM_VAL_Old){	
						if(BPM_VAL < 2) BPM_VAL = 2;
						if(BPM_VAL > 1999) BPM_VAL = 1999;
						//calcul du nombre d'instructions
						Instructions_VAL = BPM/BPM_VAL;
						BPM_VAL_Old = BPM_VAL;
						DigitConvert(BPM_VAL);		
						}			
		}	
//************Display BPM_VAL ***************
	DISP_N++;
	if(DISP_N > 2) DISP_N = 0;		 

		 switch(DISP_N){
			 case 0:
		 		DISP3 = 0;
		 		PORTD = DisplayConvert[Digit1];
				DISP1 =1;
			break;
			case 1:
				DISP1 =0;
		 		PORTD = DisplayConvert[Digit2];
				DISP2 =1;
			break;
			case 2:
				DISP2 =0;
		 		PORTD = DisplayConvert[Digit3] + Dot;
				DISP3 =1;
			break;				
			}
//*********************ISR**********************
if(Flag_INT){
	Flag_INT = 0;
		
		if(INC_Cycles >= (Instructions_VAL + Interrupt_Time + Swing_VAL)) INC_Cycles = Instructions_VAL;
		 	
		INC_Cycles -= Interrupt_Time;
				
		if(INC_Cycles <= Interrupt_Time){			
		
				switch(Beat){																
					case 1:				
					nb_Instructions = Instructions_VAL - Swing_VAL;
					break;
					case 2:				
					nb_Instructions = Instructions_VAL - Swing_VAL;
					break;																
					case 5:				
					nb_Instructions = Instructions_VAL + Swing_VAL;
					break;
					case 6:				
					nb_Instructions = Instructions_VAL + Swing_VAL;
					break;				
					default:
					nb_Instructions = Instructions_VAL;
					}											

			INC_Cycles += nb_Instructions ;
								
			Beat++;
			Beat &=0b111;
			PORT_OUT++;			
			if(PORT_OUT == 5) PORT_OUT = 1;	
					
			PORTE = PORT_OUT;
			BPM_LED = OUT_1;
			Loop_LED = 400;
		}//if (inc_cycles
		Loop_LED--;	
		Loop_BPM--;					
	
}			
			
//******************ANALOG Routine**********************
		
		if(Loop_LED == 0){ //turns the BPM LED Off
			PORTE = 0;
			BPM_LED = 0;																
		}
		
		if(Loop_BPM == 0){
			Loop_BPM = 500;				
			ANALOG_VAL = ReadCV();
			
			if(!FIVEV){
				ANALOG_VAL -= 504;
				}	
				
			if(!SWING){
				BPM_VAL = BPM_VAL_ENC + ANALOG_VAL ;
				Swing_VAL = 0;	
				}
			else{
				BPM_VAL = BPM_VAL_ENC ;				
				Swing_VAL = (long) ANALOG_VAL * Instructions_VAL/1100;
				}																			
		}								
	} //while		
}//main				
	 
	 
